; PostGIS for PostgreSQL
;
; postgis.nsi - NSIS Script for NSIS 2.0
; Requires the ExecDos plugin (http://nsis.sourceforge.net/wiki/ExecDos)
;
; Copyright (c) 2003,2004 Steffen Macke, DORSCH Consult
; Copyright (c) 2004-2005 Mark Cave-Ayland
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

; Modern User Interface
!include "MUI.nsh"

ReserveFile "${NSISDIR}\Plugins\InstallOptions.dll"
ReserveFile ".\postgis.ini"

!define UserName $R0
!define Password $R1
!define DataBase $R2
!define Port	 $R3

!define PG_VERSION "8.0"
!define GEOS_VERSION "2.2.1"
!define PROJ_VERSION "4.4.9"

!define POSTGIS_VERSION "1.1.0"
!define POSTGIS_PRODUCT "PostGIS ${POSTGIS_VERSION} for PostgreSQL ${PG_VERSION}"
!define POSTGIS_JDBCVERSION "1.1.0"
!define POSTGIS_PACKAGING "1"
!define POSTGIS_PG_VERSION "pg80"

!define POSTGIS_DIR "c:\msys\1.0\home\mca\postgis\pg80\postgis-${POSTGIS_VERSION}"
!define GEOS_DIR "c:\msys\1.0\home\mca\geos\rel-${GEOS_VERSION}"
!define PROJ_DIR "c:\msys\1.0\home\mca\proj\rel-${PROJ_VERSION}"

; --------------------------------
; Configuration

  ; General
  Name "${POSTGIS_PRODUCT}"
  OutFile "postgis-${POSTGIS_PG_VERSION}-setup-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"

  ; Folder selection page
  InstallDir "C:\Program Files\PostgreSQL\${PG_VERSION}\"
  
  ; Remember install folder
  InstallDirRegKey HKCU "Software\${POSTGIS_PRODUCT}" ""

; --------------------------------
; Interface Settings

  !define MUI_ABORTWARNING
  !define MUI_COMPONENTSPAGE_CHECKBITMAP "${NSISDIR}\Contrib\Graphics\Checks\colorful.bmp"
  !define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\box-install.ico"
  !define MUI_UNICON "${NSISDIR}\Contrib\Graphics\Icons\box-uninstall.ico"
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP "elephant.bmp"
  !define MUI_HEADERIMAGE_UNBITMAP "elephant.bmp"
  
; --------------------------------
; Pages

  !insertmacro MUI_PAGE_LICENSE "COPYING"

  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY
  Page custom SetCustom ValidateCustom ": Database Connection" 
  !insertmacro MUI_PAGE_INSTFILES
  
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES  
  
;--------------------------------
;Languages
 
  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections

Section "PostGIS" SecPostGIS
  SetShellVarContext all
  ; Checks whether installation folder is the PostgreSQL installation folder
  
  IfFileExists "$INSTDIR\bin\postgres.exe" postgres_available
  MessageBox MB_OK|MB_ICONSTOP "PostGIS has to be installed to the PostgreSQL folder."
  Abort "PostGIS has to be installed to the PostgreSQL folder."

postgres_available:  
  ; Start the installation
  SetOutPath "$INSTDIR\lib"
  File "${POSTGIS_DIR}\lwgeom\liblwgeom.dll"
  
  SetOutPath "$INSTDIR\bin"
  File "${GEOS_DIR}\bin\libgeos-2.dll"
  File "${GEOS_DIR}\bin\libgeos_c-1.dll"
  File "${PROJ_DIR}\lib\libproj.dll"

  SetOutPath "$INSTDIR\bin"
  File "${POSTGIS_DIR}\loader\pgsql2shp.exe"
  File "${POSTGIS_DIR}\loader\shp2pgsql.exe"
  
  SetOutPath "$INSTDIR\doc\contrib"
  File "${POSTGIS_DIR}\README.postgis"
  File "${POSTGIS_DIR}\loader\README.shp2pgsql"
  File "${POSTGIS_DIR}\loader\README.pgsql2shp"

  SetOutPath "$INSTDIR\doc\contrib\html"
  File "${POSTGIS_DIR}\doc\html\postgis.html"
  File "${POSTGIS_DIR}\doc\html\style.css"

  SetOutPath "$INSTDIR\share\contrib"
  File "${POSTGIS_DIR}\lwpostgis.sql"
  File "${POSTGIS_DIR}\spatial_ref_sys.sql"

  SetOutPath "$INSTDIR\share\contrib\postgis"
  File "${POSTGIS_DIR}\utils\create_undef.pl"
  File "${POSTGIS_DIR}\utils\postgis_restore.pl"
  File "${POSTGIS_DIR}\utils\postgis_proc_upgrade.pl"
  File "${POSTGIS_DIR}\lwpostgis_upgrade.sql"

  SetOutPath "$INSTDIR\jdbc"
  File "${POSTGIS_DIR}\jdbc2\postgis_${POSTGIS_JDBCVERSION}.jar"


  ; Store install folder
  WriteRegStr HKCU "Software\${POSTGIS_PRODUCT}" "" $INSTDIR
  
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayName" "${POSTGIS_PRODUCT} (remove only)"
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "UninstallString" '"$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"'
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayIcon" '"$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"'
  
  ; Create uninstaller
  WriteUninstaller "$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"

SectionEnd

Section "Create Database" SecDcmmsDb

  ReadINIStr ${UserName} "$PLUGINSDIR\postgis.ini" "Field 3" "State"
  DetailPrint "UserName=${UserName}"
  ReadINIStr ${Password} "$PLUGINSDIR\postgis.ini" "Field 5" "State"
  DetailPrint "Password=${Password}"
  ReadINIStr ${DataBase} "$PLUGINSDIR\postgis.ini" "Field 7" "State"
  DetailPrint  "DataBase=${DataBase}"
  ReadINIStr ${Port} "$PLUGINSDIR\postgis.ini" "Field 9" "State"
  DetailPrint  "Port=${Port}"

  SetOutPath "$INSTDIR\bin"

  ;File "create_postgis_db.bat"
  FileOpen $1 "$INSTDIR\bin\create_postgis_db.bat" "w"
  FileWrite $1 'set PGPASSWORD=${Password}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 'set PGPORT=${Port}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 '"$INSTDIR\bin\createdb.exe" -U ${UserName} -h localhost --encoding=UNICODE ${Database} > "$TEMP\create_postgis_db_output.txt" 2> "$TEMP\create_postgis_db_error.txt"'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileClose $1
  ExecDos::exec "/BRAND=Creating database" '"$SYSDIR\cmd.exe" /c "$INSTDIR\bin\create_postgis_db.bat"'
  FileOpen $1 "$TEMP\create_postgis_db_output.txt" "r"
  FileRead $1 $2
  FileClose $1
  FileOpen $1 "$TEMP\create_postgis_db_error.txt" "r"
  FileRead $1 $3
  StrCpy $4 $2 15
  StrCmp "$4" "CREATE DATABASE" database_created
  DetailPrint $2
  DetailPrint $3
  MessageBox MB_OK|MB_ICONSTOP 'Database creation failed: Check "$TEMP\create_postgis_db_error.txt" for detailed error messages.'
  Abort 'Database creation failed. Check "$TEMP\create_postgis_db_error.txt" for error messages "$2".'

database_created:
  Delete "$INSTDIR\bin\create_postgis_db.bat"
  Delete "$TEMP\create_postgis_db_error.txt"
  Delete "$TEMP\create_postgis_db_output.txt"
  
  FileOpen $1 "$INSTDIR\bin\create_postgis_lang.bat" "w"
  FileWrite $1 'set PGPASSWORD=${Password}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 'set PGPORT=${Port}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 '"$INSTDIR\bin\createlang.exe" -U ${UserName} -h localhost plpgsql ${Database} > "$TEMP\create_postgis_lang_output.txt" 2> "$TEMP\create_postgis_lang_error.txt"'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileClose $1
  ExecDos::exec "/BRAND=Loading PLPGSQL language" '"$SYSDIR\cmd.exe" /c "$INSTDIR\bin\create_postgis_lang.bat"'
  
  ;TODO: error checking
  Delete "$INSTDIR\bin\create_postgis_lang.bat"
  ;Delete "$TEMP\create_postgis_lang_error.txt"
  ;Delete "$TEMP\create_postgis_lang_output.txt"
  
  FileOpen $1 "$INSTDIR\bin\create_postgis_sql.bat" "w"
  FileWrite $1 'set PGPASSWORD=${Password}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 'set PGPORT=${Port}'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -f "$INSTDIR\share\contrib\lwpostgis.sql" ${Database} > "$TEMP\create_postgis_sql_output.txt" 2> "$TEMP\create_postgis_sql_error.txt"'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -f "$INSTDIR\share\contrib\spatial_ref_sys.sql" ${Database} >> "$TEMP\create_postgis_sql_output.txt" 2>> "$TEMP\create_postgis_sql_error.txt"'
  FileWriteByte $1 "13"
  FileWriteByte $1 "10"
  FileClose $1
  ExecDos::exec "/BRAND=Loading PostGIS functions" '"$SYSDIR\cmd.exe" /c "$INSTDIR\bin\create_postgis_sql.bat"'
  
  Push "500" ;character count per search
  Push "ERROR"
  Push "$TEMP\create_postgis_sql_error.txt"
  Call SearchByteFile
  Pop $0
  Pop $1 
  
  StrCmp $0 "no" postgis_loaded
  DetailPrint "$1 ERRORS"
  MessageBox MB_OK|MB_ICONSTOP 'Loading PostGIS failed. Check "$TEMP\create_postgis_sql_error.txt" for error messages.'
  Abort 'Loading PostGIS failed. Check "$TEMP\create_postgis_sql_error.txt" for error messages.'

postgis_loaded:
  Delete "$INSTDIR\bin\create_postgis_sql.bat"
  Delete "$TEMP\create_postgis_sql_error.txt"
  Delete "$TEMP\create_postgis_sql_output.txt"
     
SectionEnd

;--------------------------------
;Language Strings

LangString DESC_SecPostGIS ${LANG_ENGLISH} "Install PostGIS ${POSTGIS_VERSION} for PostgreSQL ${PG_VERSION}, GEOS ${GEOS_VERSION}, PROJ ${PROJ_VERSION}."
LangString DESC_SecDcmmsDb ${LANG_ENGLISH} "Create a database and load PostGIS into it."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecPostGIS} $(DESC_SecPostGIS)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecDcmmsDb} $(DESC_SecDcmmsDb)
!insertmacro MUI_FUNCTION_DESCRIPTION_END
 
;--------------------------------
;Uninstaller Section

Section "Uninstall"
    
  ; delete files
  Delete "$INSTDIR\lib\liblwgeom.dll"
  Delete "$INSTDIR\bin\libgeos-2.dll"
  Delete "$INSTDIR\bin\libgeos_c-1.dll"
  Delete "$INSTDIR\bin\libproj.dll"
  Delete "$INSTDIR\bin\pgsql2shp.exe"
  Delete "$INSTDIR\bin\shp2pgsql.exe"

  Delete "$INSTDIR\doc\contrib\README.postgis"
  Delete "$INSTDIR\doc\contrib\README.shp2pgsql"
  Delete "$INSTDIR\doc\contrib\README.pgsql2shp"
  Delete "$INSTDIR\doc\contrib\html\postgis.html"
  Delete "$INSTDIR\doc\contrib\html\style.css"  

  Delete "$INSTDIR\share\contrib\lwpostgis.sql"
  Delete "$INSTDIR\share\contrib\spatial_ref_sys.sql"
  
  Delete "$INSTDIR\share\contrib\postgis\create_undef.pl"
  Delete "$INSTDIR\share\contrib\postgis\postgis_restore.pl"
  Delete "$INSTDIR\share\contrib\postgis\postgis_proc_upgrade.pl"
  Delete "$INSTDIR\share\contrib\postgis\lwpostgis_upgrade.sql"
  RmDir  "$INSTDIR\share\contrib\postgis"

  Delete "$INSTDIR\jdbc\postgis_${POSTGIS_JDBCVERSION}.jar"

  DeleteRegKey /ifempty HKCU "Software\${POSTGIS_PRODUCT}"
  DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayName"
  DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "UninstallString"
  DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayIcon"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}"

  Delete "$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"
  
SectionEnd

Function .onInit
  
  InitPluginsDir
  File /oname=$PLUGINSDIR\postgis.ini ".\postgis.ini"
  
FunctionEnd

Function SetCustom
  !insertmacro MUI_HEADER_TEXT "Database Connection" "Specify the database connection"
  ;Display the InstallOptions dialog
  Push $R0

  InstallOptions::dialog $PLUGINSDIR\postgis.ini
  Pop $R0
  
FunctionEnd

Function ValidateCustom

FunctionEnd

Function SearchByteFile
Exch $0 ;file
Exch
Exch $1 ;search for
Exch
Exch 2
Exch $2 ;group-up count
Exch 2
Push $3
Push $4
Push $5
Push $6
Push $7
Push $8
Push $9

 ClearErrors
  FileOpen $9 $0 r
   IfErrors error

 StrCpy $8 0

loop_read_top:
 StrCpy $3 0
 StrCpy $5 ""

loop_read:
 ClearErrors
  FileReadByte $9 $4
   IfErrors exit
  IntFmt $4 "%c" $4
   StrCpy $5 $5$4
  IntOp $3 $3 + 1
   StrCmp $3 $2 +2
Goto loop_read

 StrLen $6 $1
 StrCpy $3 0

loop_search:
  IntOp $3 $3 - 1
  StrCpy $4 $5 $6 $3
    StrCmp $4 "" loop_read_top
StrCmp $4 $1 0 loop_search

StrCpy $0 yes
 StrCpy $5 $5 $3
 StrCpy $3 0
  IntOp $8 $8 + 1
Goto loop_search

error:
 StrCpy $0 error

exit:
 StrCmp $0 yes +3
 StrCmp $0 error +2
  StrCpy $0 no

StrCpy $1 $8
FileClose $9

Pop $9
Pop $8
Pop $7
Pop $6
Pop $5
Pop $4
Pop $3
Pop $2
Exch $1 ;found count
Exch
Exch $0 ;yes/no/error
FunctionEnd
